/* ========================================================================
 * Constants definition
 * ======================================================================== */

//The available fields. Note that not all of them is editable.
//AccessLevel, ArchiveLevel, Build, Comment, CreateDate, Description, Duration, EndDate, IssueNo, ItemType, LastEditedBy, 
//LastModifiedDate, Manager, Originator, Priority, Project, ReleaseNoticed, ReleaseScheduled, Responsible, Severity, StartDate, 
//Status, SubmitterEmail, SuperiorWorkItem, Synopsis, TargetDuration, TargetEndDate, TargetStartDate, TaskIsMilestone, WBS


//Field prefix.
var FIELD_PREFIX = 'f';

// Field constants name. Field unique names from Track+
var FIELD_NAME = {
	PROJECT: 'Project',
	ITEM_TYPE: 'ItemType',
	DESCRIPTION: 'Description',
	SYNOPSIS: 'Synopsis',
	STATUS: 'Status',
	PRIORITY: 'Priority',
	ITEM_NO: 'IssueNo',
	COMMENT: 'Comment'
}

//Field constant names with prefix.
var FIELD = {
	PROJECT: FIELD_PREFIX + FIELD_NAME.PROJECT,
	ITEM_TYPE: FIELD_PREFIX + FIELD_NAME.ITEM_TYPE,
	DESCRIPTION: FIELD_PREFIX + FIELD_NAME.DESCRIPTION,
	SYNOPSIS: FIELD_PREFIX + FIELD_NAME.SYNOPSIS,
	STATUS: FIELD_PREFIX + FIELD_NAME.STATUS,
	PRIORITY: FIELD_PREFIX + FIELD_NAME.PRIORITY,
	ITEM_NO: FIELD_PREFIX + FIELD_NAME.ITEM_NO,
	COMMENT: FIELD_PREFIX + FIELD_NAME.COMMENT
};

//The fields which are presented in table
var GRID_FIELDS = [FIELD.ITEM_NO, FIELD.SYNOPSIS, FIELD.STATUS, FIELD.COMMENT];

/**
 * This method removes all data from passed combo box.
 * @param comboULComp
 */
function removeComboValues(comboULComp) {
	if(comboULComp ) {
		comboULComp.empty();
	}
}

/**
 * This method populates the combo box passed as parameter with real data.
 * @param comboULComp
 * @param data
 * @param onComboClick
 */
function populateComboWithLabelValue(comboULComp, data, onComboClick) {
	if(comboULComp && data && data.length > 0) {
		comboULComp.empty();
		var item = '';
		for(var ind in data) {
			var dataItem = data[ind];
			item += '<li id=' + dataItem.id + '><a href="#" id="' + dataItem.id + '" >' + dataItem.label + '</a></li>';
		}
		comboULComp.append(item);		
		//After changing drop down value, set as button text the newly selected value.
		$("#" + comboULComp.attr('id') + " li a").click(function(e){
			var selText = $(this).text();
			var selID = $(this).context.id;
			$(this).parents('.dropdown').find('.dropdown-toggle').html('<span id = "' + selID + '">' + selText + '<span>');
			if(onComboClick) {
				onComboClick(selID, selText);
				$(this).parents('.dropdown').removeClass('open')
				return false;
			}else {
				$(this).parents('.dropdown').removeClass('open')
				return false;
			}
		});	
	}
}


/**
 * This method returns passed combo selected item's id.
 * @param combo
 * @returns
 */
function getSelectedComboID(combo) {
	if(combo) {
		return combo.find('.btn').find('span').first().attr('id');
	}
}

/**
 * This method selects the passed item in passed combo box.
 * @param combo
 * @param id
 * @param text
 */
function setSelectedComboID(combo, id, text) {
	var arrow = '<span class="caret"></span> ';
	if(combo) {
		combo.find('.dropdown-toggle').html('<span id = "' + id + '">' + text + '&nbsp;' + '<span>' + arrow);
	}
}