/* ========================================================================
 * Methods which use the web service
 * ======================================================================== */

/* Number of items present in grid */
var myItemsCount = 0;

/**
 * This method obtains the current user items and fills the table.
 */
function getItems() {
	var url = BASE_URL + "/items/myItems";
	var fields = {fields: GRID_FIELDS};
	setLoading(true);
	$.ajax({
		type: "POST",
        url: url,        
        async: true,
        crossDomain: true,
        data: JSON.stringify(fields),
        contentType: 'application/json; charset=utf-8',
        success: function(response, textStatus, xhr) {
        	setLoading(false);
        	if(response.success) {
        		myItemsCount = response.items.length;
        		if(response.items && response.items.length > 0) {
        			populateGridWithItems(response);
        		}else {
        			$("#jsGrid").hide();
        		}
        	}else {
        		bootbox.alert(response.error);
        	}
        },
        error:function(jqXHR, textStatus, errorThrown) {
        	setLoading(false);
        	ajaxFailure(jqXHR);
        	
        }
    });
}

/**
 * This method initializes the create item form, fills the UI components.
 * @param extraParams
 */
function initCreateItemForm(extraParams) {
	var params = {fields: []}
	params.fields.push(FIELD.PROJECT);
	params.fields.push(FIELD.ITEM_TYPE);
	params.fields.push(FIELD.STATUS);
	
	if(extraParams) {
		for(var ind in extraParams) {
			params[ind] = extraParams[ind];
		}
	}
	setLoading(true);
	$.ajax({
		type: "POST",
        url: BASE_URL + '/items/fieldsConfig',
        data: JSON.stringify(params),
        contentType: 'application/json; charset=utf-8',
        async: true,
        crossDomain: true,
        success: function(response, textStatus, xhr) {
        	setLoading(false);
        	if(response.success) {
        		populateFieldsForCreate(response);
        		initAttachmentUploader();
        	}else {
        		bootbox.alert(response.error);
        	}
        },
        error:function(jqXHR, textStatus, errorThrown) {
        	setLoading(false);
        	ajaxFailure(jqXHR);
        }
    });
}

/**
 * This method saves a new item.
 */
function save() {
	var url = BASE_URL + "/items/saveFromWebClient";
	var params = getSaveParams();
	setLoading(true);
	$.ajax({
		type: "POST",
        url: url,        
        async: true,
        crossDomain: true,
        data: JSON.stringify(params),
        contentType: 'application/json; charset=utf-8',
        success: function(response, textStatus, xhr) {
        	setLoading(false);
        	if(response.success) {
        		bootbox.alert(getText('home.newItem.created'));
        		myItemsCount++;
        		hideCreateForm();
        		getItems();
        	}else {
        		bootbox.alert(response.error);
        	}
        },
        error:function(jqXHR, textStatus, errorThrown) {
        	setLoading(false);
        	bootbox.alert(getText('common.unknownError'));
        }
    });
}

/* ========================================================================
 * Helper methods
 * ======================================================================== */

/**
 * This method localizes the Home screen.
 */
function localizeHomePage() {
//	 $("#projectsLabel").text(getText('home.fields.projects'));
//	 $("#issueTypesLabel").text(getText('home.fields.issueType'));
//	 $("#statesLabel").text(getText('home.fields.status'));
	 $("#synopsisLbl").text(getText('home.fields.title'));
	 $("#descriptionLbl").text(getText('home.fields.description'));
	 $("#addAttachmentLbl").text(getText('home.addAttachmentLbl'));
	 $("#removeAttachmentLbl").text(getText('home.removeAttachmentLbl'));
	  
	 $("#createItemCreateBtn").text(getText('home.createItemCreateBtn'));
	 $("#createItemCancelBtn").text(getText('common.cancel'));
	 
	 $("#createNewItemBtn").text(getText('home.createNewItem'));
}

/**
 * This method initializes the Home screen.
 */
function initHomeScreen() {
	setupAjax();	
	getItems();
	localizeHomePage();
	var componentLocale = 'en';
	if(PAGE_LANG) {
		var langParts = PAGE_LANG.split("_");
		if(langParts && langParts.length > 0) {
			componentLocale = langParts[0];
		}
	}
	
	$("#fileInput").fileinput({
		language: componentLocale,
		showUpload: false,
		previewFileType: 'any',
		uploadAsync: true,
		uploadUrl: BASE_URL + "/attachments/saveAttachFromWebClient"
	}).on("filebatchselected", function(event, files) {
		$("#fileInput").fileinput("upload");
	});
	
	$('#createNewItemForm').validator().on('submit', function (e) {
		if (e.isDefaultPrevented()) {
		} else {
			save();
			e.preventDefault();
		}
	});
}

function onAddAttachment() {
	$("#addAttachmentSection").fadeIn('slow');
	$("#addAttachmentLblComp").fadeOut('slow', function() {
		$("#removeAttachmentLblComp").fadeIn('slow');
	});
}

function onRemoveAttachment() {
	$("#addAttachmentSection").fadeOut('slow');
	$("#removeAttachmentLblComp").fadeOut('slow', function() {
		$("#addAttachmentLblComp").fadeIn('slow');
	});
}

/**
 * This method populates the table with columns and rows.
 * @param response
 */
function populateGridWithItems(response) {
	var gridFields = getGridFields(response.fields);
	var convertedItemsData = getGridConvertedData(response.items);
    $("#jsGrid").jsGrid({
        data: convertedItemsData,
        fields: gridFields,
        
        height: "400px",
        width: "100%",
        filtering: false,
        editing: false,
        sorting: true,
        paging: false,
        sorting: false
    });
}

/**
 * This method converts server response JSON for table.
 * @param rawDataArr
 * @returns {Array}
 */
function getGridConvertedData(rawDataArr) {
	var convertedRawDataArr = [];
	for(var ind in rawDataArr) {
		var firtsItem = rawDataArr[ind];
		var convertedItemObj = {};
		for(var itemKey in firtsItem) {
			var newItemKey = itemKey.replace(/\W/g, '');
			convertedItemObj[newItemKey] = firtsItem[itemKey];
		}
		convertedRawDataArr.push(convertedItemObj);
	}
	return convertedRawDataArr;
}

/**
 * Based on server response JSON, this method returns table fields.
 * @param fieldsRawData
 * @returns {Array}
 */
function getGridFields(fieldsRawData) {
	var gridFields = [];
	var columnIndex = 0;
	var align = "center";
	var SMALL_COL_WIDTH = 80;
	for(var ind in GRID_FIELDS) {
		var fieldName = GRID_FIELDS[ind];
		if(fieldsRawData[fieldName]) {
			var fieldItem = fieldsRawData[fieldName];
			
			if(fieldItem) {
				var width = null;
				//Width of Item. No. column
				if(columnIndex === 0) {
					width = SMALL_COL_WIDTH;
				}
				//Width of Status column
				if(columnIndex === 2) {
					width = SMALL_COL_WIDTH;
				}
				var name = fieldItem.replace(/\W/g, '');
				var title = fieldItem;
				var cfg = {
					name: name,
					title: title,
					type: "text",
					width: width,
					align: align
				};
				if(width) {
					cfg.width = width;
				}
				gridFields.push(cfg);
				columnIndex++;
			}
			
		}
	}
	return gridFields;
}

/**
 * This method fills the table with dummy fields, if the current user ha no items.
 */
function createDummyJSGrid() {
	var dummyFields = [];
	dummyFields.push({"name": "ItemNo", "title": "Item No.", "type": "text", "width": 40});
	dummyFields.push({"name": "Title", "title": "Title", "type": "text"});
	dummyFields.push({"name": "Status", "title": "Status", "type": "text"});
	dummyFields.push({"name": "Comment", "title": "Comment", "type": "text"});
	dummyFields.push({"name": "Created", "title": "Created", "type": "text"});
	var dt = [];
	$("#jsGrid").jsGrid({
        height: "400px",
        width: "100%",
        filtering: false,
        editing: false,
        sorting: true,
        paging: false,
        data: dt,//[{"name":"test"}],
        fields: dummyFields,
        sorting: false
    });
}

/**
 * This method shows create item form.
 */
function showCreateItem() {
	if(myItemsCount > 0) {
		$("#jsGrid").fadeOut('slow', function() {
			$('#newItemFromWrapper form').show('slow');
			$('#createNewItemBtn').hide('slow');
			initCreateItemForm(null);
		});
	}else {
		$('#newItemFromWrapper form').show('slow');
		$('#createNewItemBtn').hide('slow');
		initCreateItemForm(null);
	}
}

/**
 * This method hides create item form.
 */
function hideCreateForm() {
//	$('#newItemFromWrapper').animate({height: 55}, 500);
	if(myItemsCount > 0) {
		$('#newItemFromWrapper form').hide('slow', function() {
			$("#jsGrid").fadeIn();
			$('#createNewItemBtn').show('slow');
		});
	}else {
		$('#newItemFromWrapper form').hide('slow', function() {
			$('#createNewItemBtn').show('slow');
		});
	}
}

function isElementDisplayed(id) {
	if ( $("#" + id).css('display') === 'none' ) {
		return false;
	}
	return true;
}

/**
 * This method returns the given combo box selected item (id and value) 
 * @param comboJSON
 * @returns 
 */
function getComboSelectedItem(comboJSON) {
	var selectedItem = null;
	if(comboJSON && comboJSON.dataSource) {
		var dataSource = comboJSON.dataSource;
		var selectedID = comboJSON.selectedID;
		if(isNull(selectedID)) {
			if(dataSource.length > 0) {
				var firstItem = dataSource[0];
				selectedItem = {};
				selectedItem.id = firstItem.id;
				selectedItem.label = firstItem.label;
			}
		}else {
			for(var ind in dataSource) {
				if(selectedID === dataSource[ind].id) {
					selectedItem = {};
					selectedItem.id = dataSource[ind].id
					selectedItem.label = dataSource[ind].label
				}
			}
		}
	}
	return selectedItem;
}

/**
 * This method populates the create item form combo boxes with data.
 * @param response
 */
function populateFieldsForCreate(response) {
   	var projects = response[FIELD.PROJECT];
   	var ds, selectedItem;
   	if(projects && projects.dataSource) {
   		ds = projects.dataSource;
   		populateComboWithLabelValue($("#projectsUl"), ds, onProjectsComboClick);
   		selectedItem = getComboSelectedItem(projects);
   		if(selectedItem) {
   			setSelectedComboID($("#projects"), selectedItem.id, selectedItem.label);
   		}
   	}
	
	var issueTypes = response[FIELD.ITEM_TYPE];
	if(issueTypes && issueTypes.dataSource) {
		ds = issueTypes.dataSource;
		populateComboWithLabelValue($("#issueTypesUl"), ds, onIssueTypesComboClick);
		selectedItem = getComboSelectedItem(issueTypes);
		if(selectedItem) {
			setSelectedComboID($("#issueTypes"), selectedItem.id, selectedItem.label);
		}
	}
	
	var states = response[FIELD.STATUS];
	if(states && states.dataSource) {
		ds = states.dataSource;
		populateComboWithLabelValue($("#statesUL"), ds);
		selectedItem = getComboSelectedItem(states);
		if(selectedItem) {
			setSelectedComboID($('#states'), selectedItem.id, selectedItem.label);
		}
	}
}

/**
 * This method returns form values for saving an item.
 */
function getSaveParams() {
//	var projectID = getSelectedComboID($('#projects'));
//	var issueTypeID = getSelectedComboID($('#issueTypes'));
	var title = $('#synopsis').val();
	var description = $('#description').val();
//	var statusID = getSelectedComboID($('#states'));
	var fields = {};
//	fields[FIELD.PROJECT] = projectID;
//	fields[FIELD.ITEM_TYPE] = issueTypeID;
	fields[FIELD.DESCRIPTION] = description;
	fields[FIELD.SYNOPSIS] = title;
//	fields[FIELD.STATUS] = statusID;
	var params = {
		fields: fields
	}
	return params;
}

/**
 * This method handles selecting new project item. 
 * Reloads the form.
 * @param id
 * @param val
 */
function onProjectsComboClick(id, val) {
	extraParams = {
		projectID: id
	}
	initCreateItemForm(extraParams);
}

/**
 * This method handles selecting new item type. 
 * Reloads the form.
 * @param id
 * @param val
 */
function onIssueTypesComboClick(id, val) {
	var projectID = getSelectedComboID($('#projects'));
	extraParams = {
		projectID: projectID,
		issueTypeID: id
	}
	initCreateItemForm(extraParams);
}

function initAttachmentUploader() {
	$('#synopsis').val('');
	$('#description').val('');
	$('#fileInput').fileinput('refresh');
}
