$(document).ready(function() {
	checkToken();
	localizePage();
});

/**************************************************************/
/************ Methods which uses the web service **************/
/**************************************************************/

/* ========================================================================
 * Methods which use the web service
 * ======================================================================== */
/**
 * This method handles user standard authentication, (with user name and password).
 * If the authentication succeed, the method sets the tpAPIToken and tpAPILocale cookies,
 * removes login form and adds home screen (Grid and Create item form).
 */
function login() {
	var userName = $('#userName').val();
	var password = $('#password').val();
	var url = BASE_URL + "/login";
	var ePassword = encodePwd(password, "8".charCodeAt(0));
	setLoading(true);
	$.ajax({
		type: "POST",
		headers: {
        	'userName': userName,
        	'password': ePassword,
        },
        url: url,        
        async: true,
        crossDomain: true,
        success: function(response, textStatus, xhr) {
        	setLoading(false);
        	if(response.success) {
        		var token = response.token;
        		TP_LOCALE = response.locale;
        		setCookie("tpAPIToken", token);
//        		setCookie("tpAPILocale", locale);
        		setupAjax();
        		switchScreen(false);
        		initHomeScreen();
        	}else {
        		var error = response.error;
        		bootbox.alert(error);
        	}
        },
        error:function(jqXHR, textStatus, errorThrown) {
        	setLoading(false);
        	bootbox.alert(getText('common.unknownError'));
        }
    });
}

/**
 * This method handles authentication using just an E-mail address.
 * If the authentication succeed, the method sets the tpAPIToken and tpAPILocale cookies,
 * removes Login screen and adds Home screen (Grid and Create item form).
 */
function loginWithEmail() {
	var emailAddress = $('#loginWithEmailTF').val();
	var url = BASE_URL + "/loginWithEmail";
	setLoading(true);
	$.ajax({
		type: "POST",
		headers: {
        	'email': emailAddress
        },
        url: url,        
        async: true,
        crossDomain: true,
        success: function(response, textStatus, xhr) {
        	setLoading(false);
        	if(response.success) {
        		var token = response.token;
        		setCookie("tpAPIToken", token);
        		setCookie("tpAPILocale", "de_DE");
        		setupAjax();
        		switchScreen(false);
        		initHomeScreen();
        	}else {
        		var error = response.error;
        		bootbox.alert(error);
        	}
        },
        error:function(jqXHR, textStatus, errorThrown) {
        	setLoading(false);
        	bootbox.alert(getText('common.unknownError'));s
        }
    });
}

/**
 * This method handles user log out both type: standard user and user which is authenticated by E-mail.  
 */
function logout() {
	$.ajax({
		type: "POST",
        url: BASE_URL + '/logout',        
        async: true,
        crossDomain: true,
        success: function(response, textStatus, xhr) {
        	switchScreen(true);
        },
        error:function(jqXHR, textStatus, errorThrown) {
        	ajaxFailure(jqXHR);
        }
    });
}

/**
 * This method checks if actual token still valid or not.
 * If valid shows Home screen otherwise shows Login screen.
 */
function checkToken() {
	var url = BASE_URL + "/verifyToken";
	setupAjax();
	$.ajax({
		type: "GET",
        url: url,        
        async: true,
        crossDomain: true,
        success: function(response, textStatus, xhr) {
        	if(response.success) {
        		setupAjax();
        		switchScreen(false);
        		initHomeScreen();
        	}else {
        		initLoginScreen();
        	}
        },
        error:function(jqXHR, textStatus, errorThrown) {
        	initLoginScreen();
        }
    });
}

/**
 * This method handles user registration.
 */
function register() {
	var regUserName = $('#regUserName').val();
	var regPassword = $('#regPassword').val();
	var regPpasswordConfirm = $('#regPpasswordConfirm').val();
	var regEmail = $('#regEmail').val();
	var regPhone = $('#regPhone').val();
	var regLastName = $('#regLastName').val();
	var regFirstName = $('#regFirstName').val();
	var locale = getSelectedComboID($('#regLocales'));
	var params = {
		userName: regUserName,
		firstName: regFirstName,
		lastName: regLastName,
		userEmail: regEmail,
		phone: regPhone,
		passwd: regPassword,
		passwd2: regPpasswordConfirm,
		locale: locale
	}
	setLoading(true);
	var url = BASE_URL + "/register";
	$.ajax({
		type: "POST",
        url: url,        
        async: true,
        crossDomain: true,
        data: JSON.stringify(params),
        contentType: 'application/json; charset=utf-8',
        success: function(response, textStatus, xhr) {
        	if(response.success) {
        		setLoading(false);
        		$('#registrationDialog').modal('toggle');
        		bootbox.alert(getText('index.regSuccess'));
        	}else {
        		var error = response.error;
        		setLoading(false);
        		if(error) {
        			bootbox.alert(error);
        		}
        	}
        },
        error:function(jqXHR, textStatus, errorThrown) {
        	bootbox.alert(getText('common.unknownError'));
        }
    });
}

/**
 * This method handles forgot password procedure.
 */
function sendForgotPwdReq() {
	var url = BASE_URL + "/forgotPassword";
	var forgotEmail = $('#forgotEmail').val();
	var params = {
		emailAddress: forgotEmail
	}	
	setLoading(true);
	$.ajax({
		type: "POST",
        url: url,        
        async: true,
        crossDomain: true,
        data: JSON.stringify(params),
        contentType: 'application/json; charset=utf-8',
        success: function(response, textStatus, xhr) {
        	setLoading(false);
        	if(response.success) {
        		bootbox.alert(response.message);
        	}else {
        		bootbox.alert(response.error);
        	}
        },
        error:function(jqXHR, textStatus, errorThrown) {
        	setLoading(false);
        	bootbox.alert(getText('common.unknownError'));
        }
    });
}

/**
 * This method shows Registration dialog, and initializes it.
 */
function showRegDlg() {
	var url = BASE_URL + "/locales";
	$.ajax({
		type: "GET",
        url: url,        
        async: true,
        crossDomain: true,
        success: function(response, textStatus, xhr) {
        	if(response.locales) {
        		var locales = response.locales;
        		$('#registrationDialog').modal('toggle');
        		 populateComboWithLabelValue($("#regLocalesDropDownUl"), locales);
        		 
        		 if(locales && locales.length > 0) {
        			 var firstLocItem = locales[0];
        			 setSelectedComboID($("#regLocales"), firstLocItem.id, firstLocItem.label);
        		 }
        	}
        },
        error:function(jqXHR, textStatus, errorThrown) {
            bootbox.alert(getText('common.unknownError'));
        }
    });
}


/* ========================================================================
 * Helper methods
 * ======================================================================== */
/**
 * This method initializes the Login screen.
 */
function initLoginScreen() {
	localizePage();
	$('#regForm').validator().on('submit', function (e) {
		if (e.isDefaultPrevented()) {
		} else {
			register();
			e.preventDefault();
		}
	})
	
	$('#forgotForm').validator().on('submit', function (e) {
		if (e.isDefaultPrevented()) {
		} else {
			sendForgotPwdReq();
			e.preventDefault();
		}
	})
	
	$('#loginWithEmailForm').validator().on('submit', function (e) {
		if (e.isDefaultPrevented()) {
		} else {
			loginWithEmail();
			e.preventDefault();
		}
	})
	
	$('#loginWithUserAndPwdForm').validator().on('submit', function (e) {
		if (e.isDefaultPrevented()) {
		} else {
			login();
			e.preventDefault();
		}
	})
}

/**
 * This method localizes the Login screen's UI components.
 */
function localizePage() {
	
	 $("#loginBoxTitle").html(getText('index.loginBoxTitle'));
	 
	 $("#loginWithAccountLbl").html(getText('index.loginWithAccount'));
	 $("#loginWithEmailLbl").html(getText('index.loginWithEmail'));
	 $("#registerHintLbl").html(getText('index.registerHint'));
	 $("#loginBtn").text(getText('index.loginBtn'));
	 $("#loginWithEmailSubmitBtn").text(getText('index.logintWithEmailBtn'));

	 $("#loginWithEmailTF").attr("placeholder", getText('index.loginWithEmailTF'));

	 $("#userName").attr("placeholder", getText('index.userNameFieldLbl'));
	 $("#password").attr("placeholder", getText('index.userPwdFieldLbl'));
	 
	 $("#forgotPwdLink").text(getText('index.forgotPwdBtn'));
	 
	 $("#registerBtn").text(getText('index.registerBtn'));
	 
	 $("#regUserNameLabel").text(getText('index.regUserName'));
	 $("#regPasswordLabel").text(getText('index.regPassword'));
	 $("#regPasswordConfirmLabel").text(getText('index.regPasswordConfirm'));
	 $("#regPasswordConfirm").attr("data-match-error", getText('index.regPasswordNotMatch'));
	 $("#regEmailLabel").text(getText('index.regEmail'));
	 $("#regEmail").attr("data-error", getText('common.invalidEmail'));
	 $("#regPhoneLabel").text(getText('index.regPhone'));
	 $("#regLastNameLabel").text(getText('index.regLastName'));
	 $("#regFirstNameLabel").text(getText('index.regFirstName'));
	 $("#regLocalesLabel").text(getText('index.regPreferredLang'));
	 
	 $("#regSubmitBtn").text(getText('index.regSubmit'));
	 $("#regCancelBtn").text(getText('common.cancel'));
	 $("#regDlgTitle").text(getText('index.regDlgTitle'));

	 $("#forgotPwdDlgTitle").text(getText('index.forgotPwdDlgTitle'));
	 $("#forgotEmailLabel").text(getText('index.forgotPwdDlgEmail'));
	 $("#forgotSubmitBtn").text(getText('common.submit'));
	 $("#forgotEmail").attr('data-error', getText('common.invalidEmail'));

	 $("#navbarLogoutLbl").html(getText('index.logOut'));
}

/**
 * This method handles switching between Login and Home screen.
 * @param showLogin
 */
function switchScreen(showLogin) {
	if(showLogin) {
		$("#homeSection").fadeOut('slow', function() {
			$("#loginSection").fadeIn('slow');
		});
		
	}else {
		$("#loginSection").fadeOut('slow', function(){
			$("#homeSection").fadeIn('slow');
		});
	}
}

/**
 * This method handles the password encoding. 
 * @param kpw plain password
 * @param key 
 * @returns {String}
 */
function encodePwd(kpw, key) {
	var the_res = '';
    for(var i=0;i<kpw.length;++i){
        var ccode = kpw.charCodeAt(i);
        ccode = key ^ ccode;
        the_res += decimalToHex(ccode,4);
    }
    return the_res;
}

/**
 * Converts a decimal number to HEX.
 * @param d
 * @param padding
 * @returns {String}
 */
function decimalToHex(d, padding) {
    var hex = Number(d).toString(16);
    padding = isNull(padding) ? padding = 2 : padding;
    while (hex.length < padding) {
        hex = "0" + hex;
    }
    return hex;
}

/**
 * This method shows forgot password dialog.
 */
function showForgotPwd() {
	$('#forgotPwdDialog').modal('toggle');
}